package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.UserDto;
import com.qmth.distributed.print.business.entity.BasicCourse;
import com.qmth.distributed.print.business.entity.SysRole;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.entity.SysUserRole;
import com.qmth.distributed.print.business.mapper.SysUserMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.security.Md5Utils;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.business.enums.RoleTypeEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private BasicCourseService basicCourseService;

    @Autowired
    private BasicUserCourseService basicUserCourseService;

    @Autowired
    private SysConfigService sysConfigService;

    @Autowired
    private SysUserRoleService sysUserRoleService;


    @Override
    public IPage<UserDto> list(Long schoolId, String realName, String roleId, Boolean enable, Integer pageNumber, Integer pageSize) {
        Page<UserDto> page = new Page<>(pageNumber, pageSize);
        IPage<UserDto> userDtoIPage = this.baseMapper.listPage(page, schoolId, realName, roleId, enable);
        if (userDtoIPage.getRecords().size() > 0) {
            userDtoIPage.getRecords().forEach(m -> {
                //角色
                List<SysRole> roles = sysRoleService.listRolesByUserId(Long.valueOf(m.getId()));
                m.setRoles(roles);

                //课程
                List<BasicCourse> courses = basicCourseService.listCoursesByUserId(Long.valueOf(m.getId()));
                m.setCourses(courses);
            });
        }
        return userDtoIPage;
    }


    @Transactional
    @Override
    public boolean saveUser(SysUser user) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUser::getLoginName, user.getLoginName());
        SysUser sysUser = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(user.getId())) {
            if (sysUser != null) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            String password = sysConfigService.getByKey("sys.user.initPassword");
            user.setPassword(Md5Utils.toMd5Hex(StringUtils.isNoneBlank(password) ? password : "123456"));
            user.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(user);
        }
        // 修改
        else {
            if (sysUser != null && user.getId() != sysUser.getId()) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            sysUser.setRealName(user.getRealName());
            sysUser.setMobileNumber(user.getMobileNumber());
            sysUser.setOrgId(user.getOrgId());
            sysUser.setUpdateTime(System.currentTimeMillis());
            isSuccess = this.updateById(sysUser);

            //删除角色
            UpdateWrapper<SysUserRole> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(SysUserRole::getUserId, sysUser.getId());

            //删除课程
            basicUserCourseService.removeByUserId(sysUser.getId());

            sysUserRoleService.remove(updateWrapper);
        }

        //新增用户-角色
        for (Long roleId : user.getRoleIds()) {
            SysUserRole sysUserRole = new SysUserRole();
            sysUserRole.setUserId(user.getId());
            sysUserRole.setRoleId(roleId);
            sysUserRoleService.save(sysUserRole);
        }

        // 角色里是否有考务老师角色
        List<SysRole> sysRoles = sysRoleService.list(user.getRoleIds(), RoleTypeEnum.QUESTION_TEACHER.name());
        if(sysRoles != null && sysRoles.size() > 0){
            Long[] courseIds = user.getCourseIds();
            if(courseIds.length == 0){
                throw ExceptionResultEnum.ERROR.exception("请选择课程");
            }

            basicUserCourseService.saveBatch(user);

        }

        return isSuccess;
    }

    @Override
    public boolean enable(SysUser user) {
        UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysUser::getEnable, user.getEnable()).eq(SysUser::getId, user.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean resetPassword(Long id) {
        String password = sysConfigService.getByKey("sys.user.initPassword");
        String md5Password = Md5Utils.toMd5Hex(StringUtils.isNoneBlank(password) ? password : "123456");

        SysUser user = this.getById(id);
        user.setPassword(md5Password);

        return this.updateById(user);
    }

    @Override
    public boolean updatePassword(Long id, String oldPassword, String password) {
        SysUser sysUser = this.getById(id);
        if(sysUser == null){
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }

        oldPassword = Md5Utils.toMd5Hex(oldPassword);
        if(!StringUtils.equals(sysUser.getPassword(), oldPassword)){
            throw ExceptionResultEnum.ERROR.exception("原密码不正确");
        }
        String md5Password = Md5Utils.toMd5Hex(password);
        sysUser.setPassword(md5Password);
        return this.updateById(sysUser);
    }

    @Transactional
    @Override
    public boolean bindRoles(SysUser sysUser) {
        Long[] roleIds = sysUser.getRoleIds();
        if(roleIds.length == 0){
            throw ExceptionResultEnum.ERROR.exception("请选择角色");
        }
        List<SysUserRole> list = new ArrayList<>();
        for (Long roleId : roleIds) {
            SysUserRole userRole = new SysUserRole();
            userRole.setUserId(sysUser.getId());
            userRole.setRoleId(roleId);
            list.add(userRole);
        }

        // 角色里是否有考务老师角色
        List<SysRole> sysRoles = sysRoleService.list(roleIds, RoleTypeEnum.QUESTION_TEACHER.name());
        if(sysRoles != null && sysRoles.size() > 0){
            Long[] courseIds = sysUser.getCourseIds();
            if(courseIds.length == 0){
                throw ExceptionResultEnum.ERROR.exception("请选择课程");
            }

            basicUserCourseService.removeByUserId(sysUser.getId());
            basicUserCourseService.saveBatch(sysUser);

        }
        return sysUserRoleService.saveBatch(list);
    }

}
